package com.gx.obe.struct.composite;

import java.util.List;
import java.util.Map;
import java.util.OptionalInt;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.bind.prop.Prop;
import com.gx.obe.component.comparison.Column;
import com.gx.obe.component.comparison.ComparisonData;
import com.gx.obe.component.comparison.ComparisonTable;
import com.gx.obe.component.rx.RxSwt;
import com.gx.obe.component.utils.CompositeUtils;
import com.gx.obe.components.core.Constants;
//import com.gx.obe.components.core.vo.TableXML;
//import com.gx.obe.components.core.vo.TableXML.Tr;
//import com.gx.obe.components.core.vo.TableXML.Tr.Td;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.TableXML2;
import com.gx.obe.struct.beans.Td;
import com.gx.obe.struct.beans.TemplateTable;
import com.gx.obe.struct.beans.Tr;
import com.gx.obe.struct.service.TemplateTableService;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.Supplier;

public class TemplateTableComparisonComposite extends Composite {
	
	private final TemplateTableService templateTableService = new TemplateTableService();
	
	private final Prop<ComparisonData> comparisonData = new Prop<>();
	
	public TemplateTableComparisonComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new FillLayout(SWT.HORIZONTAL));
		comparisonData.bind(t -> {
			CompositeUtils.disposeChildren(this);
			new ComparisonTable(this, t);
			this.layout();
		});
	}
	
	public void refresh(String tenderId, String relChapterType, String dataCode, List<Supplier> supplierList) {
		RxSwt.run(() -> getComparisonData(tenderId, relChapterType, dataCode, supplierList)).checkWidget(this).exe(comparisonData::set);
	}
	
	/**
	 * 小范本类型数据加载
	 * @param tenderId
	 * @param relChapterType
	 * @param dataCode
	 * @param supplierList
	 * @return
	 */
	private ComparisonData getComparisonData(String tenderId, String relChapterType, String dataCode, List<Supplier> supplierList) {
		/**
		 * 小范本解析
		 */
		//获取解析入库的小范本数据
		List<TemplateTable> templateTableList = templateTableService.getTemplateTableList(tenderId, relChapterType, dataCode);
		if (CollectionUtils.isNull(templateTableList)) {
			return null;
		}
//		List<Tr> trList = null;
		int rowCount = 0;
		//获取供应商行数最多作为展示表格的行数
		Map<String, TemplateTable> templateTableMap = templateTableList.stream().collect(Collectors.toMap(TemplateTable::getSupplierId, Function.identity()));
		for(TemplateTable templateTable : templateTableList) {
			TableXML2 tableXML = JsonUtil.jsonToBean(templateTable.getTableInfo(), TableXML2.class);
			if(rowCount < tableXML.getBodyList().size()) {
				rowCount = tableXML.getBodyList().size();
			}
		}
		//获取招标文件的小范本，解析表头
		TemplateTable templateTable = templateTableMap.get(Constants.BIDDING_ID);
		if (templateTable == null) {
			return null;
		}
		//表头解析
		TableXML2 tableXML = JsonUtil.jsonToBean(templateTable.getTableInfo(), TableXML2.class);
		if (tableXML == null) {
			return null;
		}
		List<Tr> headList = tableXML.getHeadList();
		if (headList.size() != 1) {
			return null;
		}
		Tr head = headList.get(0);
		ComparisonData comparisonData = new ComparisonData();
//		for(Tr tr : tableXML.getBodyList()) {
//			System.out.println(getContent(tr, indexCol.getAsInt()));
//			comparisonData.addRow(getContent(tr, indexCol.getAsInt()));
//		}
		for(int index = 0; index < rowCount; index++) {
			comparisonData.addRow((index + 1) + "");
		}
		//表头列
		for(Supplier supplier : supplierList) {
			comparisonData.addTitleCol(new Column(supplier.getId(), supplier.getSupplierName()));
		}
		//供应商数据插入
		for(int index = 0;index < head.getTds().length; index++) {
			comparisonDataInit(templateTableMap, tableXML, head,  index, supplierList, comparisonData);
		}
		return comparisonData;
	}
	
	private void comparisonDataInit(Map<String, TemplateTable> templateTableMap, TableXML2 tableXML, Tr head,  int col, List<Supplier> supplierList, ComparisonData comparisonData) {
		Td td = head.getTds()[col];
		for(Supplier supplier : supplierList) {
			supplierComparisonDataInit(templateTableMap, col, supplier, comparisonData);
		}
		comparisonData.addItemCol(new Column(col + "", td.getContent()));

	}
	
	private void supplierComparisonDataInit(Map<String, TemplateTable> templateTableMap,  int col, Supplier supplier, ComparisonData comparisonData) {
		TemplateTable supplierTemplateTable = templateTableMap.get(supplier.getId());
		if (supplierTemplateTable == null) {
			return;
		}
		TableXML2 supplierTableXML = JsonUtil.jsonToBean(supplierTemplateTable.getTableInfo(), TableXML2.class);
//		System.out.println(supplierTemplateTable.getTableInfo());
		if (supplierTableXML == null) {
			return;
		}
//		
		int trIndex = 0;
		for(Tr  tr: supplierTableXML.getBodyList()) {
			int tdIndex = 0;
			for(Td td : tr.getTds()) {
				comparisonData.putComparisonContent(trIndex + "", supplier.getId(), tdIndex + "", td.getContent());
//				System.out.println(" put ：" + tdIndex + " : "+  supplier.getId() +" : "+  col + "; " +  td.getContent());
				tdIndex++;
				
			}
			trIndex++;
		}
//			
	}
	
	
	@Override
	protected void checkSubclass() {}
	

	
}
